@@ -406,7 +406,7 @@ range_contains_long(rangeobject *r, PyObject *ob)
     tmp2 = PyNumber_Remainder(tmp1, r->step);
     if (tmp2 == NULL)
         goto end;
-    /* result = (int(ob) - start % step) == 0 */
+    /* result = ((int(ob) - start) % step) == 0 */
     result = PyObject_RichCompareBool(tmp2, zero, Py_EQ);
   end:
     Py_XDECREF(tmp1);
@@ -937,12 +937,20 @@ rangeiter_new(PyTypeObject *type, PyObject *args, PyObject *kw)
 {
     long start, stop, step;
 
-    if (!_PyArg_NoKeywords("rangeiter()", kw))
+    if (!_PyArg_NoKeywords("range_iterator()", kw)) {
         return NULL;
+    }
 
-    if (!PyArg_ParseTuple(args, "lll;rangeiter() requires 3 int arguments",
-                          &start, &stop, &step))
+    if (!PyArg_ParseTuple(args,
+                          "lll;range_iterator() requires 3 int arguments",
+                          &start, &stop, &step)) {
         return NULL;
+    }
+    if (step == 0) {
+        PyErr_SetString(PyExc_ValueError,
+                        "range_iterator() arg 3 must not be zero");
+        return NULL;
+    }
 
     return fast_range_iter(start, stop, step);
 }
